""" Converts data to YOLOv8 format for Video Object Detection
"""
import glob, os
import xml.etree.ElementTree as ET
import argparse
import shutil


parser = argparse.ArgumentParser()
parser.add_argument("in_directory")
parser.add_argument("out_directory")

def main(args):
    # Keep track of unique object types to convert to numerical id later
    object_types = {} 
    files_written = 0
    if not os.path.exists(args.out_directory):
        os.makedirs(args.out_directory)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(args.out_directory))} files from {args.out_directory} since it exists")
        for files_ in os.listdir(args.out_directory):
                file_full_path = os.path.join(args.out_directory, files_)
                os.remove(file_full_path)

    # Find all xml files
    xml_fpaths = glob.glob('*.xml', root_dir=args.in_directory)
    print(f"Found {len(xml_fpaths)} files to convert")
    for file in xml_fpaths:
        filename = os.path.join(args.in_directory, file)
        # extract width, height
        try:
            tree = ET.parse(filename)
        except Exception:
            print(f"failed to parse {filename}")
        root = tree.getroot()
        width = int(root.find("size").find("width").text)
        height = int(root.find("size").find("height").text)
        # extract objects
        objects = root.findall("object")
        object_strings = []
        for detection in objects:
            name = detection.find("name").text
            bndbox = detection.find("bndbox")
            xmin = int(bndbox.find("xmin").text)
            xmax = int(bndbox.find("xmax").text)
            ymin = int(bndbox.find("ymin").text)
            ymax = int(bndbox.find("ymax").text)
            if name not in object_types.keys():
                object_types[name] = len(object_types)
            # Convert from min/max x/y to normalized center, width, height
            object_center_x = (xmin+xmax)/2/width
            object_center_y = (ymin+ymax)/2/height
            normalized_object_width = (xmax-xmin)/width
            normalized_object_height = (ymax-ymin)/height
            object_strings.append(f"{object_types[name]} {object_center_x} {object_center_y} {normalized_object_width} {normalized_object_height}")
        if len(object_strings) > 0:
            with open(os.path.join(args.out_directory,(file[:-4]+".txt")),"w") as f:
                f.write("\n".join(object_strings))
                files_written += 1
    print(f"Wrote {files_written} files for images with detections")
    
    #now delete the labels_ folder which had the xmls
    print(f"deleting {args.in_directory}")
    shutil.rmtree(args.in_directory)



if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
