""" Take in split by source and then we will
make xml into txt files
and then split image files by fish and non fish.
This will then prepare for a train/ val split
"""
import glob, os
import argparse
import re
import shutil
import xml.etree.ElementTree as ET



parser = argparse.ArgumentParser()
parser.add_argument("in_directory_labels")
parser.add_argument("in_directory_images")
parser.add_argument("txt_file_location")
parser.add_argument("out_directory_images")
parser.add_argument("number_fishless_to_include")

#this defines our classes and determines where images can go in the split
SOURCES = {"fish":[], "no_fish": []}


def main(args):
    
    #####convert xmls to txts in labels
    # Keep track of unique object types to convert to numerical id later
    object_types = {} 
    files_written = 0
    if not os.path.exists(args.txt_file_location):
        os.makedirs(args.txt_file_location)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(args.txt_file_location))} files from {args.txt_file_location} since it exists")
        for files_ in os.listdir(args.txt_file_location):
                file_full_path = os.path.join(args.txt_file_location, files_)
                os.remove(file_full_path)
    # Find all xml files
    xml_fpaths = glob.glob("*.xml", root_dir=args.in_directory_labels)
    print(f"Found {len(xml_fpaths)} files to convert in {args.in_directory_labels}")
    for file in xml_fpaths:
        filename = os.path.join(args.in_directory_labels, file)
        # extract width, height
        try:
            tree = ET.parse(filename)
        except Exception:
            print(f"failed to parse {filename}")
        root = tree.getroot()
        width = int(root.find("size").find("width").text)
        height = int(root.find("size").find("height").text)
        # extract objects
        objects = root.findall("object")
        object_strings = []
        for detection in objects:
            name = detection.find("name").text
            bndbox = detection.find("bndbox")
            xmin = int(bndbox.find("xmin").text)
            xmax = int(bndbox.find("xmax").text)
            ymin = int(bndbox.find("ymin").text)
            ymax = int(bndbox.find("ymax").text)
            if name not in object_types.keys():
                object_types[name] = len(object_types)
            # Convert from min/max x/y to normalized center, width, height
            object_center_x = (xmin+xmax)/2/width
            object_center_y = (ymin+ymax)/2/height
            normalized_object_width = (xmax-xmin)/width
            normalized_object_height = (ymax-ymin)/height
            object_strings.append(f"{object_types[name]} {object_center_x} {object_center_y} {normalized_object_width} {normalized_object_height}")
        if len(object_strings) > 0:
            with open(os.path.join(args.txt_file_location,(file[:-4]+".txt")),"w") as f:
                f.write("\n".join(object_strings))
                files_written += 1
    print(f"Wrote {files_written} files for images with detections.")
    
    

    #### separate fish from non fish in images
    image_directory = args.in_directory_images
    image_fpaths = os.listdir(image_directory)
    #now image_fpaths are all the image files
    image_lists = {k:[] for k in SOURCES}
    for fpath in image_fpaths:
        #for each file in the list, check if the file path exists in the .txt folder
        fpath_file_name = fpath[:-4] + ".txt"
        #make file name for hypothetical txt file that will only exist if the image has a fish in it
        fpath_txt_version = os.path.join(args.txt_file_location, fpath_file_name)
        #now check if that txt file exists
        if not os.path.exists(fpath_txt_version):
            image_lists['no_fish'].append(fpath) #if the file does not exist, then we add to the "no_fish" folder
        else:
            image_lists['fish'].append(fpath) #else if the txt file does exist, the image has a fish and we add to 'fish' folder
        
    print(f"Found {len(image_fpaths)} images.")

    
    #source will be 'fish' and then 'no_fish', but we only care about fish images
    # so we will only create the new folder for the files in the 'fish' list item and not the 'no fish' one
    #before this was a dictionary, and now it is just one of them because we dont care about saving the no fish images in this case
    source = 'fish'
    img_out_dir = os.path.join(args.out_directory_images)
    #make folder if it doesn't exist
    if not os.path.exists(img_out_dir):
        os.makedirs(img_out_dir)
    else: #this means there are files already there, so we want to remove them
        for files_ in os.listdir(img_out_dir):
                file_full_path = os.path.join(img_out_dir, files_)
                os.remove(file_full_path)
        
    for fpath in image_lists[source]:
        shutil.copy(os.path.join(image_directory, fpath), os.path.join(img_out_dir, fpath))
    #this is where we will add in a few non fish images to the output image folder
    #I will do it so there is an input of how many fishless images to add (X), and we will select the first X from the fishless vector
    print(f"We will add {args.number_fishless_to_include} fishless images from the list of {len(image_lists['no_fish'])} total fishless images.")
    for fishless_image_fpath in image_lists['no_fish'][0:int(args.number_fishless_to_include)]:
        shutil.copy(os.path.join(image_directory, fishless_image_fpath), os.path.join(img_out_dir, fishless_image_fpath))
            
    print(f"Copied {len(image_lists['fish'])} images for fish.")
    print(f"Copied {args.number_fishless_to_include} images for no fish.")

    #remove the original image and labels folders for storage reasons
    print(f"deleting {image_directory}")
    shutil.rmtree(image_directory)
    print(f"deleting {args.in_directory_labels}")
    shutil.rmtree(args.in_directory_labels)

    

if __name__ == "__main__":
    args = parser.parse_args()
    main(args)