""" Split combined dataset by video source, and also split Igiugig
"""
import glob, os
import argparse
import re
import shutil

parser = argparse.ArgumentParser()
parser.add_argument("datasets_directory")
parser.add_argument("dataset_name")
parser.add_argument("out_directory")
SOURCES = {"Igiugig_1": (lambda s: (s[0].isdigit()) & (s[16]=="1")),
           "Igiugig_2": (lambda s: (s[0].isdigit()) & (s[16]=="2")),
           "Igiugig_3": (lambda s: (s[0].isdigit()) & (s[16]=="3")),
            "Voith": lambda s: s.startswith("Voith"), "WellsDam": lambda s: s.startswith("WellsDam")}


def main(args):
    dataset_directory = os.path.join(args.datasets_directory, args.dataset_name)
    image_directory = os.path.join(dataset_directory, "train_x")
    label_directory = os.path.join(dataset_directory, "train_y")
    
    image_fpaths = os.listdir(image_directory)
    image_lists = {k:[] for k in SOURCES}
    for fpath in image_fpaths:
        for k, f in SOURCES.items():
            if f(fpath): 
                image_lists[k].append(fpath)
                break
    label_fpaths = os.listdir(label_directory)
    label_lists = {k:[] for k in SOURCES}
    print(f"Found {len(image_fpaths)} images.")
    print(f"Found {len(label_fpaths)} labels.")

    for fpath in label_fpaths:
        for k, f in SOURCES.items():
            if f(fpath): 
                label_lists[k].append(fpath)
                break
    for source in SOURCES:
        img_out_dir = os.path.join(args.out_directory, source, "images_first_iter")
        label_out_dir = os.path.join(args.out_directory, source, "labels_first_iter")
        for dir_name in [img_out_dir, label_out_dir]:
            if not os.path.exists(dir_name):
                os.makedirs(dir_name)
            else:
                print(f"removing {len(os.listdir(dir_name))} files from {dir_name} since it exists")
                #this means the folder existed so we should delete everything from it
                for files_ in os.listdir(dir_name):
                    file_full_path = os.path.join(dir_name, files_)
                    os.remove(file_full_path)
        for fpath in image_lists[source]:
            shutil.copy(os.path.join(image_directory, fpath), os.path.join(img_out_dir, fpath))
        print(f"Copied {len(image_lists[source])} images for {source}.")
        for fpath in label_lists[source]:
            shutil.copy(os.path.join(label_directory, fpath), os.path.join(label_out_dir, fpath))
        print(f"Copied {len(label_lists[source])} labels for {source}.")

    #now remove the train_x and train_y folders
    print(f"deleting {image_directory}")
    shutil.rmtree(image_directory)
    print(f"deleting {label_directory}")
    shutil.rmtree(label_directory)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)