""" Split combined dataset by video source, and also split Igiugig, adn removing the duplicate images within Igiugig
"""
import glob, os
import argparse
import re
import shutil
import cv2
import numpy as np

parser = argparse.ArgumentParser()
parser.add_argument("datasets_directory")
parser.add_argument("dataset_name")
parser.add_argument("out_directory")
SOURCES = {"Igiugig_1": (lambda s: (s[0].isdigit()) & (s[16]=="1")),
           "Igiugig_2": (lambda s: (s[0].isdigit()) & (s[16]=="2")),
           "Igiugig_3": (lambda s: (s[0].isdigit()) & (s[16]=="3")),
            "Voith": lambda s: s.startswith("Voith"), "WellsDam": lambda s: s.startswith("WellsDam")}


def main(args):
    dataset_directory = os.path.join(args.datasets_directory, args.dataset_name)
    image_directory = os.path.join(dataset_directory, "train_x")
    label_directory = os.path.join(dataset_directory, "train_y")
    
    image_fpaths = os.listdir(image_directory)
    image_lists = {k:[] for k in SOURCES}
    for fpath in image_fpaths:
        for k, f in SOURCES.items():
            if f(fpath): 
                image_lists[k].append(fpath)
                break
    label_fpaths = os.listdir(label_directory)
    label_lists = {k:[] for k in SOURCES}
    print(f"Found {len(image_fpaths)} images.")
    print(f"Found {len(label_fpaths)} labels.")

    for fpath in label_fpaths:
        for k, f in SOURCES.items():
            if f(fpath): 
                label_lists[k].append(fpath)
                break
    for source in SOURCES:
        img_out_dir = os.path.join(args.out_directory, source, "images_first_iter")
        label_out_dir = os.path.join(args.out_directory, source, "labels_first_iter")
        for dir_name in [img_out_dir, label_out_dir]:
            if not os.path.exists(dir_name):
                os.makedirs(dir_name)
            else:
                print(f"removing {len(os.listdir(dir_name))} files from {dir_name} since it exists")
                #this means the folder existed so we should delete everything from it
                for files_ in os.listdir(dir_name):
                    file_full_path = os.path.join(dir_name, files_)
                    os.remove(file_full_path)
        for fpath in image_lists[source]:
            shutil.copy(os.path.join(image_directory, fpath), os.path.join(img_out_dir, fpath))
        print(f"Copied {len(image_lists[source])} images for {source}.")
       
        for fpath in label_lists[source]:
            shutil.copy(os.path.join(label_directory, fpath), os.path.join(label_out_dir, fpath))
        print(f"Copied {len(label_lists[source])} labels for {source}.")

        #now within each folder, order the items
        image_directory_for_non_dup = os.path.join(img_out_dir)
        image_names_for_non_dup = os.listdir(image_directory_for_non_dup)
        print(f"the first few image names before sorting are {image_names_for_non_dup[0:3]}")
        image_names_for_non_dup.sort(key=lambda f: int(re.sub(r'\D', '', f)))
        print(f"the first few image names after sorting are {image_names_for_non_dup[0:3]}")
        print(f"fish source is {source}")
        labels_dir_before_de_dup = os.listdir(label_out_dir)
        

        num_img_duplicate = 0
        num_labels_removed = 0
        #now we will check to see if each image is a duplicate of the other ones 
        for index_, current_img_to_comp in enumerate(image_names_for_non_dup[:-1]):
            full_img_path_to_check = os.path.join(img_out_dir, current_img_to_comp)
            #check if this image is the same as the next one
            full_img_next_in_line = os.path.join(img_out_dir, image_names_for_non_dup[index_+1])
            current_image_asarray = cv2.imread(full_img_path_to_check)
            next_image_asarray = cv2.imread(full_img_next_in_line)
            if current_image_asarray.shape==next_image_asarray.shape and (current_image_asarray ==next_image_asarray).all():
                os.remove(full_img_path_to_check)
                #now go to the label out directory and see if the label exists, if it does, delete that for the duplicate as well
                label_base_file_name = current_img_to_comp[:-4] + ".xml" #remove .png which is 4 characters
                label_full_fpath = os.path.join(label_out_dir, label_base_file_name)
                #now check if that txt file exists
                if os.path.exists(label_full_fpath):#if there is a label for the duplicate image, bc we have identified it is a duplicate if it's in this if statement, then remove it
                    os.remove(label_full_fpath) #if the file does not exist, then we add to the "no_fish" folder
                    num_labels_removed += 1
                
                num_img_duplicate += 1
        print(f"total number of images for the {source} fish source is {len(image_names_for_non_dup)} and number of duplicates was {num_img_duplicate} and new number of files in oputput directory is {len(os.listdir(img_out_dir))}")
        print(f"total number of labels for the {source} fish source is {len(labels_dir_before_de_dup)} and number of duplicates was {num_labels_removed} and new number of labels in oputput directory is {len(os.listdir(label_out_dir))}")


    #now remove the train_x and train_y folders
    print(f"deleting {image_directory}")
    shutil.rmtree(image_directory)
    print(f"deleting {label_directory}")
    shutil.rmtree(label_directory)

    

if __name__ == "__main__":
    args = parser.parse_args()
    main(args)