""" Split combined dataset by video source

NEW USERS: customize all parser and hard coded paths to data or models

"""
import glob, os
import argparse
import re
import shutil

def restricted_float(x):
    try:
        x = float(x)
    except ValueError:
        raise argparse.ArgumentTypeError("%r not a floating-point literal" % (x,))

    if x < 0.0 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 1.0]"%(x,))
    return x

parser = argparse.ArgumentParser()
parser.add_argument("in_directory") #base "/Users/victoriasabo/Documents/WPTO/Data"
parser.add_argument("in_image_folder_name") #images_/fish   orpc_test_x
parser.add_argument("in_label_folder_name") #labels_        orpc_test_y_txt
parser.add_argument("out_image_folder_name") #images/fish   images
parser.add_argument("out_label_folder_name") #labels        labels_
parser.add_argument("out_directory") #base                  /Users/victoriasabo/Documents/eyesea-yolo/scripts/data/val_and_test_files"
parser.add_argument("dataset_name") #name of source         orpc
parser.add_argument("ratio", type=restricted_float, help="Proportion of data to use for validation; .3 = 30% val")

def video_from_filename(fname):
    return fname.split("_")[:-1]
def frame_from_filename(fname):
    return int(fname.split(".")[0].split("_")[-1])

def main(args):
    image_directory = os.path.join(args.in_directory, args.in_image_folder_name) 
    label_directory = os.path.join(args.in_directory, args.in_label_folder_name)
    image_fpaths = sorted(os.listdir(image_directory), key=lambda fpath: (video_from_filename(fpath), frame_from_filename(fpath)))
    num_images = len(image_fpaths)
    label_fpaths = sorted(os.listdir(label_directory), key=lambda fpath: (video_from_filename(fpath), frame_from_filename(fpath)))
    print(f"Found {num_images} images.")
    print(f"Found {len(label_fpaths)} labels.")
    num_val = int(num_images*args.ratio)
    #check that image chosen is not in the middle of consecutive frames 
    first_image_for_test = image_fpaths[num_val]
    last_image_for_val = image_fpaths[num_val-1]

    #now check that there is not a fish (i.e., a corresponding txt file) in both of these,
    #otherwise you will have to iterate so youre not splitting the same fish being tracked
    first_image_test_txt = first_image_for_test[:-4] + ".txt"
    first_image_fpath_txt_version = os.path.join(args.in_directory, args.in_label_folder_name, first_image_test_txt)
    if not os.path.exists(first_image_fpath_txt_version):
        first_img_has_fish = False #if the file does not exist, then the first image does not have a fish
        #and we can technically make this cut right here because they are not contiguous frames
    else:
        first_img_has_fish = True #this image has a fish and so if the other has a fish, we need to choose a different split
        # because this would mean that continugous frames have fish
    
    last_image_test_txt = last_image_for_val[:-4] + ".txt"
    last_image_fpath_txt_version = os.path.join(args.in_directory, args.in_label_folder_name, last_image_test_txt)
    if not os.path.exists(last_image_fpath_txt_version):
        last_img_has_fish = False #if the file does not exist, then the first image does not have a fish
        #and we can technically make this cut right here because they are not contiguous frames
    else:
        last_img_has_fish = True #this image has a fish and so if the other has a fish, we need to choose a different split
        # because this would mean that continugous frames have fish
    

    #parse into filenames for each of these
    first_tr_left_part = first_image_for_test.split("_fm_")[0]
    last_val_left_part = last_image_for_val.split("_fm_")[0]
    first_tr_right_part = int(first_image_for_test.split("_fm_")[1][:-4])#take off the .png
    last_val_right_part = int(last_image_for_val.split("_fm_")[1][:-4])#take of .png to be left with just the number
    
    #while loop will test if left parts are equal and numbers subtract to give <3 difference,
    #if this happens, that means we have continuous frames and we shouldn't split on continuous frame
    #unless there are not fish in both of them
    #which is why there is a second condition to the while that checks if both of the frames to be split on have fish
    #if they both do not have fish, we are ok to split
    #the only time we need to choose another 2 frames to split on would be if the frames are continuous and they both have tracked fish
    while (first_tr_left_part==last_val_left_part and abs(first_tr_right_part-last_val_right_part)<=3) and (first_img_has_fish==True and last_img_has_fish==True):
        #change the first and last image
        num_val +=1
        first_image_for_test = image_fpaths[num_val]
        last_image_for_val = image_fpaths[num_val-1]
        #parse into filenames for each of these
        first_tr_left_part = first_image_for_test.split("_fm_")[0]
        last_val_left_part = last_image_for_val.split("_fm_")[0]
        first_tr_right_part = int(first_image_for_test.split("_fm_")[1][:-4])
        last_val_right_part = int(last_image_for_val.split("_fm_")[1][:-4])

        first_image_test_txt = first_image_for_test[:-4] + ".txt"
        first_image_fpath_txt_version = os.path.join(args.in_directory, args.in_label_folder_name, first_image_test_txt)
        if not os.path.exists(first_image_fpath_txt_version):
            first_img_has_fish = False #if the file does not exist, then the first image does not have a fish
            #and we can technically make this cut right here because they are not contiguous frames
        else:
            first_img_has_fish = True #this image has a fish and so if the other has a fish, we need to choose a different split
            # because this would mean that continugous frames have fish
    
        last_image_test_txt = last_image_for_val[:-4] + ".txt"
        last_image_fpath_txt_version = os.path.join(args.in_directory, args.in_label_folder_name, last_image_test_txt)
        if not os.path.exists(last_image_fpath_txt_version):
            last_img_has_fish = False #if the file does not exist, then the first image does not have a fish
            #and we can technically make this cut right here because they are not contiguous frames
        else:
            last_img_has_fish = True #this image has a fish and so if the other has a fish, we need to choose a different split
            # because this would mean that continugous frames have fish
        

    #now we have a good val/ test set split
    print(f"Selecting first {num_val} images for validation set")
    val_image_fpaths = image_fpaths[:num_val]
    test_image_fpaths = image_fpaths[num_val:]

    last_val_image = val_image_fpaths[-1]
    last_val_video = video_from_filename(last_val_image)
    last_val_image_frame = frame_from_filename(last_val_image)
    # Iterate through labels until we pass the last image
    found_video = False
    for i, fname in enumerate(label_fpaths):
        video = video_from_filename(fname)
        frame = frame_from_filename(fname)
        if found_video:
            if (not video == last_val_video) or frame > last_val_image_frame:
                first_test_label_index = i
                break
        elif video == last_val_video:
            found_video = True
            if frame > last_val_image_frame:
                first_test_label_index = i
                break
    # if not first_test_label_index:
    #     raise Exception("Couldn't find any labels for validation set (Maybe ratio is too low?)")
    #its a possibility there will be no labeled data in val set for some of smaller ones (igiugig)
    print(f"Selecting first {first_test_label_index} labels for validation set")
    val_label_fpaths = label_fpaths[:first_test_label_index]
    test_label_fpaths = label_fpaths[first_test_label_index:]
    #changed folder name so there aren't 2 copies of images in same folder
    test_img_out = os.path.join(args.out_directory, args.dataset_name, "test",
                                 args.out_image_folder_name)
    val_img_out = os.path.join(args.out_directory, args.dataset_name, "val",
                               args.out_image_folder_name)
    test_label_out = os.path.join(args.out_directory, args.dataset_name, "test",
                                   args.out_label_folder_name)
    val_label_out = os.path.join(args.out_directory, args.dataset_name, "val",
                                 args.out_label_folder_name)

    for directory in [test_img_out, val_img_out, test_label_out, val_label_out]:
        if not os.path.exists(directory):
            os.makedirs(directory)
        else: #this means there are files already there, so we want to remove them
            print(f"removing {len(os.listdir(directory))} files from {directory} since it exists")
            for files_ in os.listdir(directory):
                    file_full_path = os.path.join(directory, files_)
                    os.remove(file_full_path)

    for fpath in test_image_fpaths:
        shutil.copy(os.path.join(image_directory, fpath), os.path.join(test_img_out, fpath))
    for fpath in val_image_fpaths:
        shutil.copy(os.path.join(image_directory, fpath), os.path.join(val_img_out, fpath))   
    for fpath in test_label_fpaths:
        shutil.copy(os.path.join(label_directory, fpath), os.path.join(test_label_out, fpath))
    for fpath in val_label_fpaths:
        shutil.copy(os.path.join(label_directory, fpath), os.path.join(val_label_out, fpath))
    
    #delete the orpc_test_x folder
    print(f"deleting test fish images from original folder, filepath {image_directory}")
    shutil.rmtree(image_directory)
    #delete the orpc_test_y folder
    print(f"deleting test fish labels from original folder, filepath {label_directory}")
    shutil.rmtree(label_directory)
    


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)