""" Split combined dataset by video source
"""
import glob, os
import argparse
import re
import shutil
import cv2
import numpy as np

def restricted_float(x):
    try:
        x = float(x)
    except ValueError:
        raise argparse.ArgumentTypeError("%r not a floating-point literal" % (x,))

    if x < 0.0 or x > 1.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 1.0]"%(x,))
    return x

parser = argparse.ArgumentParser()
parser.add_argument("in_directory") #base "/Users/victoriasabo/Documents/WPTO/Data"
parser.add_argument("in_image_folder_name") #images_/fish   orpc_test_x
parser.add_argument("in_label_folder_name") #labels_        orpc_test_y_txt
parser.add_argument("out_image_folder_name") #images/fish   images
parser.add_argument("out_label_folder_name") #labels        labels
parser.add_argument("out_directory") #base                  /Users/victoriasabo/Documents/eyesea-yolo/scripts/data/val_and_test_files"
parser.add_argument("dataset_name") #name of source         orpc
parser.add_argument("ratio", type=restricted_float, help="Proportion of data to use for validation; .3 = 30% val")

def video_from_filename(fname):
    return fname.split("_")[:-1]
def frame_from_filename(fname):
    return int(fname.split(".")[0].split("_")[-1])

def main(args):
    image_directory = os.path.join(args.in_directory, args.in_image_folder_name) 
    label_directory = os.path.join(args.in_directory, args.in_label_folder_name)
    image_fpaths = sorted(os.listdir(image_directory), key=lambda fpath: (video_from_filename(fpath), frame_from_filename(fpath)))
    num_images = len(image_fpaths)
    label_fpaths = sorted(os.listdir(label_directory), key=lambda fpath: (video_from_filename(fpath), frame_from_filename(fpath)))
    print(f"Found {num_images} images.")
    print(f"Found {len(label_fpaths)} labels.")

    #now order the image folder
    #now within each folder, order the items
    image_names_for_non_dup = os.listdir(image_directory)
    print(f"the first few image names before sorting are {image_names_for_non_dup[0:13]} for the {args.dataset_name}")
    image_names_for_non_dup.sort(key=lambda f: int(re.sub(r'\D', '', f)))
    print(f"the first few image names after sorting are {image_names_for_non_dup[0:13]}")

    label_names_for_non_dup = os.listdir(label_directory)
    print(f"the first few label names before sorting are {label_names_for_non_dup[0:13]} for the {args.dataset_name}")
    label_names_for_non_dup.sort(key=lambda f: int(re.sub(r'\D', '', f)))
    print(f"the first few label names after sorting are {label_names_for_non_dup[0:13]}")

    print(f"fish source is {args.dataset_name}")
    labels_dir_before_de_dup = os.listdir(label_directory)
    

    num_img_duplicate = 0
    num_labels_removed = 0
    #now we will check to see if each image is a duplicate of the other ones 
    for index_, current_img_to_comp in enumerate(image_names_for_non_dup[:-1]):
        full_img_path_to_check = os.path.join(image_directory, current_img_to_comp)
        #check if this image is the same as the next one
        full_img_next_in_line = os.path.join(image_directory, image_names_for_non_dup[index_+1])
        current_image_asarray = cv2.imread(full_img_path_to_check)
        next_image_asarray = cv2.imread(full_img_next_in_line)
        if current_image_asarray.shape==next_image_asarray.shape and (current_image_asarray ==next_image_asarray).all():
            os.remove(full_img_path_to_check)
            #now go to the label out directory and see if the label exists, if it does, delete that for the duplicate as well
            label_base_file_name = current_img_to_comp[:-4] + ".txt" #remove .png which is 4 characters
            label_full_fpath = os.path.join(label_directory, label_base_file_name)
            #now check if that txt file exists
            if os.path.exists(label_full_fpath):#if there is a label for the duplicate image, bc we have identified it is a duplicate if its in this if statement, then remove it
                os.remove(label_full_fpath) #if the file does not exist, then we add to the "no_fish" folder
                num_labels_removed += 1
            
            num_img_duplicate += 1
    print(f"total number of images for the {args.dataset_name} fish source is {len(image_names_for_non_dup)} and number of duplicates was {num_img_duplicate} and new number of files in oputput directory is {len(os.listdir(image_directory))}")
    print(f"total number of labels for the {args.dataset_name} fish source is {len(labels_dir_before_de_dup)} and number of duplicates was {num_labels_removed} and new number of labels in oputput directory is {len(os.listdir(label_directory))}")

    #now re-do the fpath lists bc we have editted the amount if images:
    image_fpaths = os.listdir(image_directory)
    image_fpaths.sort(key=lambda f: int(re.sub(r'\D', '', f)))
    num_images = len(image_fpaths)
    label_fpaths = os.listdir(label_directory)
    label_fpaths.sort(key=lambda f: int(re.sub(r'\D', '', f)))
    num_labels = len(label_fpaths)
    print(f"Found {num_images} images post de-dup.")
    print(f"Found {num_labels} labels post de-dup.")


#switching so it is indexed on splitting the number of labels between images
    num_val = int(num_labels*args.ratio)
    #check that image chosen is not in the middle of consecutive frames 
    first_label_for_test = label_fpaths[num_val]
    last_label_for_val = label_fpaths[num_val-1]

    #now check that these are not right next to each other, otherwise they are consecutive fish

    #we dont need to check that the txt images exist, so we'll just test that some are not iterative
    # so that there are no consecutive fish split between val and test

    #parse into filenames for each of these
    first_tr_left_part = first_label_for_test.split("_fm_")[0]
    last_val_left_part = last_label_for_val.split("_fm_")[0]
    first_tr_right_part = int(first_label_for_test.split("_fm_")[1][:-4])#take off the .png
    last_val_right_part = int(last_label_for_val.split("_fm_")[1][:-4])#take of .png to be left with just the number
    print(f"name of {first_label_for_test} was split into {first_tr_left_part} and {first_tr_right_part}")
    print(f"name of {last_label_for_val} was split into {last_val_left_part} and {last_val_right_part}")
    
    #while loop will test if left parts are equal and numbers subtract to give <7 difference,
    #if this happens, that means we have continuous frames and we shouldn't split on continuous frame
    #unless there are not fish in both of them
    #which is why there is a second condition to the while that checks if both of the frames to be split on have fish
    #if they both do not have fish, we are ok to split
    #the only time we need to choose another 2 frames to split on would be if the frames are continuous and they both have tracked fish
    while (first_tr_left_part==last_val_left_part and abs(first_tr_right_part-last_val_right_part)<=7):
        #change the first and last image
        num_val +=1
        first_label_for_test = label_fpaths[num_val]
        last_label_for_val = label_fpaths[num_val-1]
        #parse into filenames for each of these
        first_tr_left_part = first_label_for_test.split("_fm_")[0]
        last_val_left_part = last_label_for_val.split("_fm_")[0]
        first_tr_right_part = int(first_label_for_test.split("_fm_")[1][:-4])
        last_val_right_part = int(last_label_for_val.split("_fm_")[1][:-4])
        print(f"name of {first_label_for_test} was split into {first_tr_left_part} and {first_tr_right_part}")
        print(f"name of {last_label_for_val} was split into {last_val_left_part} and {last_val_right_part}")

        

    #now we have a good val/ test set split
    print(f"Selecting first {num_val} labels for validation set")
    val_label_fpaths = label_fpaths[:num_val]
    test_label_fpaths = label_fpaths[num_val:]

    last_val_label = val_label_fpaths[-1]
    last_val_image_name = last_val_label[:-4] + ".png"
    index_of_last_image = image_fpaths.index(last_val_image_name)
    print(f"Selecting first {index_of_last_image+1} images for validation set")
    val_image_fpaths = image_fpaths[:index_of_last_image +1]
    test_image_fpaths = image_fpaths[index_of_last_image +1:]
    
    #changed folder name so there aren't 2 copies of images in same folder
    test_img_out = os.path.join(args.out_directory, args.dataset_name, "test",
                                 args.out_image_folder_name)
    val_img_out = os.path.join(args.out_directory, args.dataset_name, "val",
                               args.out_image_folder_name)
    test_label_out = os.path.join(args.out_directory, args.dataset_name, "test",
                                   args.out_label_folder_name)
    val_label_out = os.path.join(args.out_directory, args.dataset_name, "val",
                                 args.out_label_folder_name)

    for directory in [test_img_out, val_img_out, test_label_out, val_label_out]:
        if not os.path.exists(directory):
            os.makedirs(directory)
        else: #this means there are files already there, so we want to remove them
            print(f"removing {len(os.listdir(directory))} files from {directory} since it exists")
            for files_ in os.listdir(directory):
                    file_full_path = os.path.join(directory, files_)
                    os.remove(file_full_path)

    for fpath in test_image_fpaths:
        shutil.copy(os.path.join(image_directory, fpath), os.path.join(test_img_out, fpath))
    for fpath in val_image_fpaths:
        shutil.copy(os.path.join(image_directory, fpath), os.path.join(val_img_out, fpath))   
    for fpath in test_label_fpaths:
        shutil.copy(os.path.join(label_directory, fpath), os.path.join(test_label_out, fpath))
    for fpath in val_label_fpaths:
        shutil.copy(os.path.join(label_directory, fpath), os.path.join(val_label_out, fpath))
    
    #delete the orpc_test_x folder
    print(f"deleting test fish images from original folder, filepath {image_directory}")
    shutil.rmtree(image_directory)
    #delete the orpc_test_y folder
    print(f"deleting test fish labels from original folder, filepath {label_directory}")
    shutil.rmtree(label_directory)
    


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)